@@ -1,11 +1,13 @@
 /* GENIUS Calculator
- * Copyright (C) 1997-2005 Jiri (George) Lebl
+ * Copyright (C) 1997-2010 Jiri (George) Lebl
  *
  * Author: Jiri (George) Lebl
  *
- * This program is free software; you can redistribute it and/or modify
+ * This file is part of Genius.
+ *
+ * Genius is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -14,9 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the  Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
- * USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /*
@@ -47,6 +47,8 @@
 
 #include "genius-i18n.h"
 
+#include "binreloc.h"
+
 #include <readline/readline.h>
 #include <readline/history.h>
 
@@ -62,10 +64,14 @@
 #include <termcap.h>
 #endif
 
+#include <vicious.h>
+
 /*Globals:*/
 
+const gboolean genius_is_gui = FALSE;
+
 /*calculator state*/
-calcstate_t curstate={
+GelCalcState curstate={
 	128,
 	12,
 	FALSE,
@@ -73,15 +79,22 @@ calcstate_t curstate={
 	FALSE,
 	5,
 	TRUE,
-	10
+	10,
+	0, /* output_style */
+	0, /* max_nodes */ /* FIXME: implement here just like in gnome-genius */
+	20, /* chop */
+	5 /* chop_when */
 	};
+
+const GelHookFunc gel_evalnode_hook = NULL;
+const GelHookFunc _gel_tree_limit_hook = NULL;
 	
 extern int parenth_depth;
 
-extern int interrupted;
-
 static int use_readline = TRUE;
 
+static gboolean genius_in_dev_dir = FALSE;
+
 static int errors_printed = 0;
 static long total_errors_printed = 0;
 
@@ -120,9 +133,143 @@ gel_printout_infos (void)
 void
 gel_call_help (const char *function)
 {
-	/* No extra processing of help */
+	char *file;
+	char *str;
+
+	if (function != NULL) {
+		/* No extra processing of help,
+		 * printout was done in funclib */
+		return;
+	}
+
+	/*fp = fopen ("../doc/genius.txt", "r");
+	if G_LIKELY (fp == NULL)*/
+
+	str = gbr_find_data_dir (DATADIR);
+	file = g_build_filename (str, "genius", "genius.txt", NULL);
+	g_free (str);
+
+	if G_UNLIKELY (access (file, R_OK) != 0) {
+		g_free (file);
+		file = g_build_filename (DATADIR, "genius", "genius.txt", NULL);
+		if G_UNLIKELY (access (file, R_OK) != 0) {
+			puterror (_("Cannot locate the manual"));
+			g_free (file);
+			return;
+		}
+	}
+
+	str = g_find_program_in_path ("less");
+	if (str == NULL)
+		str = g_find_program_in_path ("more");
+
+	if G_LIKELY (str != NULL) {
+		char *argv[3];
+
+		argv[0] = str;
+		argv[1] = file;
+		argv[2] = NULL;
+		g_spawn_sync  (NULL /* wd */,
+			       argv,
+			       NULL /* envp */,
+			       0 /* flags */,
+			       NULL /* child_setup */,
+			       NULL /* user_data */,
+			       NULL /*stdout*/,
+			       NULL /*stderr*/,
+			       NULL /* status */,
+			       NULL /* error */);
+	} else {
+		/* No less or more, hence just type out the
+		 * manual */
+		FILE *fp;
+
+		fp = fopen (file, "r");
+		if G_LIKELY (fp != NULL) {
+			char buf[256];
+			while (fgets (buf, sizeof(buf), fp) != NULL) {
+				g_print ("%s\n", buf);
+			}
+
+			fclose (fp);
+		}
+	}
+	g_free (file);
 }
 
+char *
+gel_ask_string (const char *query, const char *def)
+{
+	char *txt = NULL;
+
+	g_print ("\n%s\n", ve_sure_string (query));
+	if (use_readline) {
+		char *s;
+		if ( ! ve_string_empty (def)) {
+			const char *p;
+			for (p = def; *p != '\0'; p++)
+				rl_stuff_char (*p);
+		}
+		s = readline (">");
+		if (s != NULL) {
+			txt = g_strdup (s);
+			free (s);
+		}
+	} else {
+		char buf[256];
+		if ( ! ve_string_empty (def))
+			g_print (_("Suggested: %s\n"), ve_sure_string (def));
+		if (fgets (buf, sizeof (buf), stdin) != NULL) {
+			int len = strlen (buf);
+			if (buf[len-1] == '\n')
+				buf[len-1] = '\0';
+			txt = g_strdup (buf);
+		}
+	}
+	return txt;
+}
+
+int
+gel_ask_buttons (const char *query, GSList *buttons)
+{
+	int ret;
+	GSList *li;
+	int i;
+	int max;
+
+reread_buttons:
+	g_print ("\n%s\n", ve_sure_string (query));
+	i = 1;
+	for (li = buttons; li != NULL; li = li->next) {
+		g_print ("%d) %s\n", i, ve_sure_string ((char *)li->data));
+		i++;
+	}
+	max = i-1;
+	if (use_readline) {
+		char *s;
+		s = readline (">");
+		ret = -1;
+		if ( ! ve_string_empty (s)) {
+			if (sscanf (s, "%d", &ret) != 1) {
+				ret = -1;
+			}
+		}
+	} else {
+		char buf[256];
+		ret = -1;
+		if (fgets (buf, sizeof (buf), stdin) != NULL) {
+			if (sscanf (buf, "%d", &ret) != 1) {
+				ret = -1;
+			}
+		}
+	}
+	if (ret == 0 || ret > max) {
+		g_print (_("Out of range!\n"));
+		goto reread_buttons;
+	}
+
+	return ret;
+}
 
 static int
 long_get_term_width (void)
@@ -150,8 +297,8 @@ get_term_width (GelOutput *gelo)
 }
 
 
-static void
-set_state(calcstate_t state)
+void
+gel_set_state(GelCalcState state)
 {
 	curstate = state;
 
@@ -159,15 +306,15 @@ set_state(calcstate_t state)
 	    state.output_style == GEL_OUTPUT_LATEX ||
 	    state.output_style == GEL_OUTPUT_MATHML ||
 	    state.output_style == GEL_OUTPUT_TROFF)
-		gel_output_set_length_limit (main_out, FALSE);
+		gel_output_set_length_limit (gel_main_out, FALSE);
 	else
-		gel_output_set_length_limit (main_out, TRUE);
+		gel_output_set_length_limit (gel_main_out, TRUE);
 }
 
 static void
 interrupt (int sig)
 {
-	interrupted = TRUE;
+	gel_interrupted = TRUE;
 	if (use_readline)
 		rl_stuff_char ('\n');
 	signal (SIGINT, interrupt);
@@ -198,8 +345,24 @@ main(int argc, char *argv[])
 	gboolean do_compile = FALSE;
 	gboolean do_gettext = FALSE;
 	gboolean be_quiet = FALSE;
+	char *exec = NULL;
 
-	genius_is_gui = FALSE;
+	g_set_prgname ("genius");
+	g_set_application_name (_("Genius"));
+
+	/* kind of a hack to find out if we are being run from the
+	 * directory we were built in */
+	file = g_get_current_dir ();
+	if (file != NULL &&
+	    strcmp (file, BUILDDIR "/src") == 0 &&
+	    access ("genius.c", F_OK) == 0 &&
+	    access ("../lib/lib.cgel", F_OK) == 0) {
+		genius_in_dev_dir = TRUE;
+	} else {
+		genius_in_dev_dir = FALSE;
+		gbr_init (NULL);
+	}
+	g_free (file);
 
 	/* Hmmm, everything in UTF-8? */
 	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
@@ -210,8 +373,6 @@ main(int argc, char *argv[])
 
 	signal (SIGINT, interrupt);
 
-	statechange_hook = set_state;
-
 	for(i=1;i<argc;i++) {
 		int val;
 		if(lastarg || argv[i][0]!='-')
@@ -225,6 +386,15 @@ main(int argc, char *argv[])
 				val = 128;
 			}
 			curstate.float_prec = val;
+		} else if (strcmp (argv[i], "--precision")==0 && i+1 < argc) {
+			val = 0;
+			sscanf (argv[++i],"%d",&val);
+			if (val < 60 || val > 16384) {
+				g_printerr (_("%s should be between %d and %d, using %d"),
+					    "--precision", 60, 16384, 128);
+				val = 128;
+			}
+			curstate.float_prec = val;
 		} else if(sscanf(argv[i],"--maxdigits=%d",&val)==1) {
 			if (val < 0 || val > 256) {
 				g_printerr (_("%s should be between %d and %d, using %d"),
@@ -232,6 +402,15 @@ main(int argc, char *argv[])
 				val = 12;
 			}
 			curstate.max_digits = val;
+		} else if (strcmp (argv[i], "--maxdigits")==0 && i+1 < argc) {
+			val = -1;
+			sscanf (argv[++i],"%d",&val);
+			if (val < 0 || val > 256) {
+				g_printerr (_("%s should be between %d and %d, using %d"),
+					    "--maxdigits", 0, 256, 12);
+				val = 12;
+			}
+			curstate.max_digits = val;
 		} else if(strcmp(argv[i],"--floatresult")==0)
 			curstate.results_as_floats = TRUE;
 		else if(strcmp(argv[i],"--nofloatresult")==0)
@@ -251,13 +430,38 @@ main(int argc, char *argv[])
 				val = 5;
 			}
 			curstate.max_errors = val;
+		} else if (strcmp (argv[i], "--maxerrors")==0 && i+1 < argc) {
+			val = -1;
+			sscanf (argv[++i],"%d",&val);
+			if (val < 0) {
+				g_printerr (_("%s should be greater then or equal to %d, using %d"),
+					    "--maxerrors", 0, 5);
+				val = 5;
+			}
+			curstate.max_errors = val;
 		} else if(strcmp(argv[i],"--mixed")==0)
 			curstate.mixed_fractions = TRUE;
 		else if(strcmp(argv[i],"--nomixed")==0)
 			curstate.mixed_fractions = FALSE;
-		else if(sscanf(argv[i],"--intoutbase=%d",&val)==1)
+		else if(sscanf(argv[i],"--intoutbase=%d",&val)==1) {
 			curstate.integer_output_base = val;
-		else if(strcmp(argv[i],"--readline")==0)
+		} else if (strcmp (argv[i], "--intoutbase")==0 && i+1 < argc) {
+			val = 10;
+			sscanf (argv[++i],"%d",&val);
+			curstate.integer_output_base = val;
+		} else if(sscanf(argv[i],"--chop=%d",&val)==1) {
+			curstate.integer_output_base = val;
+		} else if (strcmp (argv[i], "--chop")==0 && i+1 < argc) {
+			val = 20;
+			sscanf (argv[++i],"%d",&val);
+			curstate.chop = val;
+		} else if(sscanf(argv[i],"--chopwhen=%d",&val)==1) {
+			curstate.integer_output_base = val;
+		} else if (strcmp (argv[i], "--chopwhen")==0 && i+1 < argc) {
+			val = 10;
+			sscanf (argv[++i],"%d",&val);
+			curstate.chop_when = val;
+		} else if(strcmp(argv[i],"--readline")==0)
 			use_readline = TRUE;
 		else if(strcmp(argv[i],"--noreadline")==0)
 			use_readline = FALSE;
@@ -273,11 +477,15 @@ main(int argc, char *argv[])
 			be_quiet = TRUE;
 		else if(strcmp(argv[i],"--noquiet")==0)
 			be_quiet = FALSE;
-		else if (strcmp (argv[i], "--version") == 0) {
+		else if (strncmp (argv[i], "--exec=", strlen ("--exec=")) == 0) {
+			exec = g_strdup ((argv[i])+strlen("--exec="));
+		} else if (strcmp (argv[i], "--exec") && i+1 < argc) {
+			exec = g_strdup (argv[++i]);
+		} else if (strcmp (argv[i], "--version") == 0) {
 			g_print (_("Genius %s\n"
 				   "%s%s\n"),
 				 VERSION,
-				 COPYRIGHT_STRING,
+				 _(GENIUS_COPYRIGHT_STRING),
 				 get_version_details ());
 			exit (0);
 		} else {
@@ -297,12 +505,15 @@ main(int argc, char *argv[])
 				   "\t--maxerrors=num   \tMaximum errors to display (0=no limit) [5]\n"
 				   "\t--[no]mixed       \tPrint fractions in mixed format\n"
 				   "\t--intoutbase=num  \tBase to use to print out integers [10]\n"
+				   "\t--chop=num        \tChop small numbers less than 10^-num [20]\n"
+				   "\t--chopwhen=num    \tBut only when other numbers 10^-num or more [5]\n"
 				   "\t--[no]readline    \tUse readline if it is available [ON]\n"
 				   "\t--[no]compile     \tCompile everything and dump it to stdout [OFF]\n"
 				   "\t--[no]gettext     \tDump help/error strings in fake .c file to\n"
 				   "\t                  \tstdout (for use with gettext) [OFF]\n"
 				   "\t--[no]quiet       \tBe quiet during non-interactive mode,\n"
-				   "\t                  \t(always on when compiling) [OFF]\n\n"),
+				   "\t                  \t(always on when compiling) [OFF]\n"
+				   "\t--exec=expr       \tExecute an expression\n\n"),
 				 VERSION);
 			if (strcmp (argv[i], "--help") != 0)
 				exit (1);
@@ -311,26 +522,16 @@ main(int argc, char *argv[])
 		}
 	}
 
-#if 0
-	{
-		/* FIXME: use this for option parsing,
-		 * we really only need gnome-program for
-		 * gnome_config, perhaps we should switch to
-		 * ve-config anyway */
-		char *fakeargv[] = { argv[0], NULL };
-		gnome_program_init ("genius", VERSION, 
-				    LIBGNOME_MODULE /* module_info */,
-				    1, fakeargv,
-				    /* GNOME_PARAM_POPT_TABLE, options, */
-				    NULL);
+	if (files != NULL && exec != NULL) {
+		g_printerr (_("Can't specify both an expression and files to execute on the command line"));
+		exit (1);
 	}
-#endif
 
 	gel_read_plugin_list();
 
 	if (do_compile || do_gettext)
 		be_quiet = TRUE;
-	inter = isatty(0) && !files && !(do_compile || do_gettext);
+	inter = isatty(0) && !files && !exec && !(do_compile || do_gettext);
 	/*interactive mode, print welcome message*/
 	if (inter) {
 		g_print (_("Genius %s\n"
@@ -339,22 +540,22 @@ main(int argc, char *argv[])
 			   "For license details type `warranty'.\n"
 			   "For help type 'manual' or 'help'.%s\n\n"),
 			 VERSION,
-			 COPYRIGHT_STRING,
+			 _(GENIUS_COPYRIGHT_STRING),
 			 get_version_details ());
 		be_quiet = FALSE;
 	}
 
-	main_out = gel_output_new();
+	gel_main_out = gel_output_new();
 	if(!be_quiet)
-		gel_output_setup_file(main_out, stdout, 80,
+		gel_output_setup_file(gel_main_out, stdout, 80,
 				      get_term_width);
 	else
-		gel_output_setup_black_hole(main_out);
+		gel_output_setup_black_hole(gel_main_out);
 
 
-	set_new_calcstate(curstate);
-	set_new_errorout(calc_puterror);
-	set_new_infoout(puterror);
+	gel_set_new_calcstate(curstate);
+	gel_set_new_errorout(calc_puterror);
+	gel_set_new_infoout(puterror);
 
 	/*init the context stack and clear out any stale dictionaries
 	  except the global one, if this is the first time called it
@@ -362,23 +563,41 @@ main(int argc, char *argv[])
 	  dictionary*/
 	d_singlecontext ();
 
+	gel_init ();
+
 	if ( ! (do_compile || do_gettext)) {
 		/*
 		 * Read main library
 		 */
-		if (access ("../lib/lib.cgel", F_OK) == 0) {
+		if (genius_in_dev_dir) {
 			/*try the library file in the current/../lib directory*/
 			gel_load_compiled_file (NULL, "../lib/lib.cgel", FALSE);
 		} else {
+			char *datadir = gbr_find_data_dir (DATADIR);
+			char *file = g_build_filename (datadir,
+						       "genius",
+						       "gel",
+						       "lib.cgel",
+						       NULL);
+			if (access (file, F_OK) != 0) {
+				g_free (file);
+				file = g_build_filename (DATADIR,
+							 "genius",
+							 "gel",
+							 "lib.cgel",
+							 NULL);
+			}
 			gel_load_compiled_file (NULL,
-						LIBRARY_DIR "/gel/lib.cgel",
+						file,
 						FALSE);
+			g_free (file);
+			g_free (datadir);
 		}
 
 		/*
 		 * Read init files
 		 */
-		file = g_strconcat(g_getenv("HOME"),"/.geniusinit",NULL);
+		file = g_build_filename (g_get_home_dir (), ".geniusinit",NULL);
 		if(file)
 			gel_load_file(NULL, file, FALSE);
 		g_free(file);
@@ -417,11 +636,15 @@ main(int argc, char *argv[])
 				gel_save_plugins ();
 			return 0;
 		}
+	} else if (exec != NULL) {
+		fp = NULL;
+		gel_push_file_info("expr",1);
 	} else {
 		fp = stdin;
 		gel_push_file_info(NULL,1);
 	}
-	gel_lexer_open(fp);
+	if (fp != NULL)
+		gel_lexer_open(fp);
 	if(inter && use_readline) {
 		init_inter();
 	}
@@ -430,6 +653,14 @@ main(int argc, char *argv[])
 	
 	rl_event_hook = nop;
 
+	if (exec != NULL) {
+		line_len_cache = -1;
+		gel_evalexp (exec, NULL, gel_main_out, NULL, FALSE, NULL);
+		gel_test_max_nodes_again ();
+		line_len_cache = -1;
+		goto after_exec;
+	}
+
 	for(;;) {
 		for(;;) {
 			if(inter && use_readline) /*use readline mode*/ {
@@ -438,13 +669,14 @@ main(int argc, char *argv[])
 				line_len_cache = -1;
 				e = get_p_expression();
 				line_len_cache = -1;
-				if(e) gel_evalexp_parsed(e,main_out,"= ",TRUE);
+				if(e) gel_evalexp_parsed(e,gel_main_out,"= ",TRUE);
+				gel_test_max_nodes_again ();
 				line_len_cache = -1;
 			} else {
 				line_len_cache = -1;
-				gel_evalexp(NULL, fp, main_out, NULL, FALSE, NULL);
+				gel_evalexp(NULL, fp, gel_main_out, NULL, FALSE, NULL);
 				line_len_cache = -1;
-				if (interrupted)
+				if G_UNLIKELY (gel_interrupted)
 					gel_got_eof = TRUE;
 			}
 			if(inter)
@@ -473,36 +705,20 @@ main(int argc, char *argv[])
 				}
 			} while(!fp && files);
 			if(!fp && !files) {
-				if (do_compile) {
-					gel_push_file_info(NULL,0);
-					gel_compile_all_user_funcs(stdout);
-					gel_pop_file_info();
-					/* if we have gotten errors then
-					   signal by returning a 1 */
-					if(total_errors_printed)
-						return 1;
-				} else if (do_gettext) {
-					gel_push_file_info (NULL, 0);
-					gel_dump_strings_from_help (stdout);
-					gel_dump_strings_from_user_funcs (stdout);
-					gel_pop_file_info ();
-					/* if we have gotten errors then
-					   signal by returning a 1 */
-					if (total_errors_printed)
-						return 1;
-				} else {
-					gel_save_plugins ();
-				}
-				return 0;
+				goto after_exec;
 			}
 			gel_lexer_open(fp);
 		} else
 			break;
 	}
 
+after_exec:
+	gel_test_max_nodes_again ();
+
 	gel_printout_infos ();
 	
-	gel_lexer_close(fp);
+	if (fp != NULL)
+		gel_lexer_close(fp);
 	/*if(fp != stdin)
 		fclose(fp);*/
 	
